@@ -40,6 +40,8 @@ struct _GrdRdpGfxSurface
   uint16_t width;
   uint16_t height;
 
+  gboolean is_auxiliary_surface;
+
   GrdRdpGfxSurface *render_surface;
 
   GrdRdpGfxFrameController *frame_controller;
@@ -83,6 +85,12 @@ grd_rdp_gfx_surface_get_height (GrdRdpGfxSurface *gfx_surface)
   return gfx_surface->height;
 }
 
+gboolean
+grd_rdp_gfx_surface_is_auxiliary_surface (GrdRdpGfxSurface *gfx_surface)
+{
+  return gfx_surface->is_auxiliary_surface;
+}
+
 gboolean
 grd_rdp_gfx_surface_disallows_hwaccel_sessions (GrdRdpGfxSurface *gfx_surface)
 {
@@ -105,6 +113,7 @@ grd_rdp_gfx_surface_override_render_surface (GrdRdpGfxSurface *gfx_surface,
   g_assert (!gfx_surface->render_surface);
 
   gfx_surface->render_surface = render_surface;
+  render_surface->is_auxiliary_surface = TRUE;
 }
 
 GrdRdpGfxFrameController *
@@ -147,8 +156,8 @@ grd_rdp_gfx_surface_new (GrdRdpGraphicsPipeline           *graphics_pipeline,
     }
   else
     {
-      gfx_surface->width = gfx_surface->rdp_surface->width;
-      gfx_surface->height = gfx_surface->rdp_surface->height;
+      gfx_surface->width = grd_rdp_surface_get_width (gfx_surface->rdp_surface);
+      gfx_surface->height = grd_rdp_surface_get_height (gfx_surface->rdp_surface);
     }
 
   grd_rdp_graphics_pipeline_create_surface (graphics_pipeline, gfx_surface);
