@@ -570,8 +570,7 @@ refresh_gfx_surface_avc420 (GrdRdpGraphicsPipeline *graphics_pipeline,
   gboolean pending_bw_measure_stop = FALSE;
   int i;
 
-  if (!rdp_surface->valid)
-    rdp_surface->valid = TRUE;
+  rdp_surface->valid = TRUE;
 
   aligned_width = surface_width + (surface_width % 16 ? 16 - surface_width % 16 : 0);
   aligned_height = surface_height + (surface_height % 64 ? 64 - surface_height % 64 : 0);
@@ -871,12 +870,9 @@ refresh_gfx_surface_rfx_progressive (GrdRdpGraphicsPipeline *graphics_pipeline,
     }
 
   graphics_pipeline->rfx_context->mode = RLGR1;
-  if (!rdp_surface->valid)
-    {
-      rfx_context_reset (graphics_pipeline->rfx_context,
-                         surface_width, surface_height);
-      rdp_surface->valid = TRUE;
-    }
+  rfx_context_reset (graphics_pipeline->rfx_context,
+                     surface_width, surface_height);
+  rdp_surface->valid = TRUE;
 
   codec_context_id = grd_rdp_gfx_surface_get_codec_context_id (gfx_surface);
   g_mutex_lock (&graphics_pipeline->gfx_mutex);
@@ -983,20 +979,39 @@ refresh_gfx_surface_rfx_progressive (GrdRdpGraphicsPipeline *graphics_pipeline,
 
 static void
 map_surface_to_output (GrdRdpGraphicsPipeline *graphics_pipeline,
-                       GrdRdpGfxSurface       *gfx_surface)
+                       GrdRdpGfxSurface       *gfx_surface,
+                       GrdRdpSurfaceMapping   *surface_mapping)
 {
   RdpgfxServerContext *rdpgfx_context = graphics_pipeline->rdpgfx_context;
-  RDPGFX_MAP_SURFACE_TO_OUTPUT_PDU map_surface_to_output = {0};
-  GrdRdpSurface *rdp_surface = grd_rdp_gfx_surface_get_rdp_surface (gfx_surface);
+  GrdRdpSurfaceMappingType mapping_type = surface_mapping->mapping_type;
   uint16_t surface_id = grd_rdp_gfx_surface_get_surface_id (gfx_surface);
+  RDPGFX_MAP_SURFACE_TO_OUTPUT_PDU map_surface_to_output = {};
+
+  g_assert (mapping_type == GRD_RDP_SURFACE_MAPPING_TYPE_MAP_TO_OUTPUT);
 
   map_surface_to_output.surfaceId = surface_id;
-  map_surface_to_output.outputOriginX = rdp_surface->output_origin_x;
-  map_surface_to_output.outputOriginY = rdp_surface->output_origin_y;
+  map_surface_to_output.outputOriginX = surface_mapping->output_origin_x;
+  map_surface_to_output.outputOriginY = surface_mapping->output_origin_y;
 
   rdpgfx_context->MapSurfaceToOutput (rdpgfx_context, &map_surface_to_output);
 }
 
+static void
+map_surface (GrdRdpGraphicsPipeline *graphics_pipeline,
+             GrdRdpGfxSurface       *gfx_surface)
+{
+  GrdRdpSurface *rdp_surface = grd_rdp_gfx_surface_get_rdp_surface (gfx_surface);
+  GrdRdpSurfaceMapping *surface_mapping;
+
+  surface_mapping = grd_rdp_surface_get_mapping (rdp_surface);
+  switch (surface_mapping->mapping_type)
+    {
+    case GRD_RDP_SURFACE_MAPPING_TYPE_MAP_TO_OUTPUT:
+      map_surface_to_output (graphics_pipeline, gfx_surface, surface_mapping);
+      break;
+    }
+}
+
 static void
 clear_old_enc_times (GrdRdpGraphicsPipeline *graphics_pipeline,
                      int64_t                 current_time_us)
@@ -1096,13 +1111,11 @@ grd_rdp_graphics_pipeline_refresh_gfx (GrdRdpGraphicsPipeline *graphics_pipeline
 
       rdp_surface->gfx_surface = grd_rdp_gfx_surface_new (graphics_pipeline,
                                                           &surface_descriptor);
-      frame_controller =
-        grd_rdp_gfx_frame_controller_new (session_rdp,
-                                          graphics_pipeline->pipeline_context,
-                                          rdp_surface);
+      frame_controller = grd_rdp_gfx_frame_controller_new (session_rdp,
+                                                           rdp_surface);
       grd_rdp_gfx_surface_attach_frame_controller (rdp_surface->gfx_surface,
                                                    g_steal_pointer (&frame_controller));
-      map_surface_to_output (graphics_pipeline, rdp_surface->gfx_surface);
+      map_surface (graphics_pipeline, rdp_surface->gfx_surface);
     }
 
   surface_id = grd_rdp_gfx_surface_get_surface_id (rdp_surface->gfx_surface);
