@@ -24,7 +24,7 @@
 #include <glib-object.h>
 #include <stdint.h>
 
-#include "grd-dbus-screen-cast.h"
+#include "grd-dbus-mutter-screen-cast.h"
 #include "grd-types.h"
 
 #define GRD_TYPE_STREAM (grd_stream_get_type ())
@@ -35,13 +35,18 @@ struct _GrdStreamClass
   GObjectClass parent_class;
 };
 
+GrdStream *grd_stream_new (uint32_t                        stream_id,
+                           GrdDBusMutterScreenCastStream  *proxy,
+                           GError                        **error);
+
+void grd_stream_destroy (GrdStream *stream);
+
+uint32_t grd_stream_get_stream_id (GrdStream *stream);
+
 uint32_t grd_stream_get_pipewire_node_id (GrdStream *stream);
 
-const char * grd_stream_get_object_path (GrdStream *stream);
+const char * grd_stream_get_mapping_id (GrdStream *stream);
 
 void grd_stream_disconnect_proxy_signals (GrdStream *stream);
 
-GrdStream * grd_stream_new (GrdContext              *context,
-                            GrdDBusScreenCastStream *proxy);
-
 #endif /* GRD_STREAM_H */
